<?php
/*
Plugin Name: Advanced Client Manager 
Plugin URI: http://codecanyon.net/user/lakedesign_nl
Description: A powerful plugin that meets your needs as a small business or freelancer. It is extremely easy to use and you can manage your clients in a straightforward manner. It has all the essential features to manage customers and their contacts, projects and invoices. All of this together in one environment to produce better results. Furthermore the plugin is fully integrated in the default WordPress theme which creates an ultimate user experience.
Version: 1.4.1
Author: Koen Lakeman
Author URI: http://www.lakedesign.nl
License: Commercial (http://codecanyon.net/licenses)
*/
?>
<?php
// Reporting E_NOTICE can be good too (to report uninitialized
// variables or catch variable name misspellings ...)


//avoid direct calls to this file where wp core files not present
if (!function_exists ('add_action')) {
	header('Status: 403 Forbidden');
	header('HTTP/1.1 403 Forbidden');
	exit();
}

if(!function_exists('wp_get_current_user')) {
    include(ABSPATH . "wp-includes/pluggable.php"); 
}
if(!function_exists('wp_delete_user')) {
    include(ABSPATH . "wp-admin/includes/user.php"); 
}

include ('api.php');
include ('clientportal.php');
include ('ipnhandler.php');

class upf{
	
	function __construct($page) {
		global $screen;
		//init variables
		$this->plugin_option_name = 'upf_advanced_client_manager';
		$this->path = plugin_dir_path(__FILE__);
		$this->dir = plugins_url('',__FILE__);
		$this->version = '1.4.1';
		$this->page = $page;	
		$this->page['url'] = admin_url('admin.php?page='.$page['slug']);
		
		register_activation_hook( __FILE__, array( &$this, 'on_activation'));		
		
		if(is_array($page)){
			//register callback to setup the admin menu
			add_action('admin_menu', array( &$this, 'setup_admin_menu') );

			if(isset($_GET['page'])){
				if($_GET['page'] == $page['slug'] ) {

					//init active content
					$this->setup_child($this->page['children']);
					$this->setup_tab($this->child['tabs']);
					$this->setup_item();
					
					//add filter because wordpress 2.8 and lower does not contains 'screen_layout_columns'
					add_filter('screen_layout_columns', array(&$this, 'setup_layout_columns'), 10, 2);

					//init classses
					$this->setup_boxes();
					$this->setup_fields();		

					//register the callback been used if options of page been submitted and needs to be processed
					if(isset($_POST['submit']) && !empty($_POST)){
						$this->on_page_save();
					}
				}
			}
		}	
		
	}
	
	function setup_child($children) {
		if(is_array($children)){
			$active = false;
			foreach ($children as $child) {
				// select the active child
				if(!$active && !empty($_GET['child'])){
					if($_GET['child'] == $child['slug']){
						$this->child = $child; // save the active child
						$active = true;
					}
				}elseif(!$active){			
					$this->child = $child; // save the active child
					$active = true;
				}
			}
		}
	}	

	function setup_tab($tabs) {
		if(is_array($tabs)){
			$active = false;
			foreach ($tabs as $tab) {
				// select the active tab
				if(!$active && !empty($_GET['tab'])){
					if($_GET['tab'] == $tab['slug']){
						$this->tab = $tab; // save the active tab
						$active = true;
					}
				}elseif(!$active){			
					$this->tab = $tab; // save the active tab
					$active = true;
				}
			}
		}
	}	

	function setup_item() {
		if(isset($_GET['item'])){
			$this->item['id'] = esc_html($_GET['item']);
		}else{
			$this->item['id'] = "1"; // set first item
			$_GET['item'] = "1";
		}	

		if(!ctype_digit($_GET['item']) && !empty($_GET['item']) || isset($_GET['item']) && empty($_GET['item'])){
			$message = 'There is no valid item specified: <strong>'.esc_html($_GET['item']).'</strong>';
			wp_die($message);
		}
		
		$_GET['action'] = isset($_GET['action']) ? $_GET['action'] : '';
		switch ($_GET['action']) {
			case "new":
				$options = upf_get_options($this->create_prefix('child','_'),'object');
				if(is_array($options)){
					$items = array();
					foreach($options as $option){
						$items[] = $option->item_id;
					}
					$latest_item = max($items);
					
					$this->item['id'] = $latest_item + 1; // set new item	
					$_GET['item'] = $this->item['id'];
				}
			break;
			case "delete":
				$action = $this->create_prefix('item').'delete';
				$nonce = esc_html($_GET['_wpnonce']);				
				
				if(!isset($_GET['item'])){
					$message = 'There isn\'t an item specified.';
					wp_die($message);
				}elseif(!wp_verify_nonce($nonce, $action)){
					$message = 'Nonce security check failed.';
					wp_die($message);				
				}
				$option = $this->create_prefix('item','_');		
				$this->on_page_delete($option);				
			break;
			case "edit":
			default:
				// everything is already done..
			break;
		}
	}		

	
	function setup_boxes(){
		$dir = $this->path . 'boxes/';	
		include_once($dir.'box.php');
	
		// include all the php-files width the box classes once
		$ext = 'php';	
		$files = glob($dir.'*.'.$ext);
		if(is_array($files)){
			foreach ($files as $file) {
				include_once($file);
				$name = basename($file, '.'.$ext);
				$boxes[$name] = new $name($this); 
			}
		}
		$this->boxes = $boxes;
	}

	function setup_fields(){
		$dir = $this->path . 'fields/';
		include_once($dir.'field.php');

		// include all the php-files width the field classes once
		$ext = 'php';	
		$files = glob($dir.'*.'.$ext);
		if(is_array($files)){
			foreach ($files as $file) {
				include_once($file);
				$name = basename($file, '.'.$ext);
				$fields[$name] = new $name($this); 
			}
		}
		$this->fields = $fields;
	}
	
	//for WordPress 2.8 we have to tell, that we support 2 columns !
    function setup_layout_columns( $columns, $screen ) {
		if(is_multisite()){
				$columns[ $this->pagehook ] = 1;
		}else{
			if($this->tab['layout_columns'] < 2){
				unset($columns[ $this->pagehook ]);
			}else{
				$columns[ $this->pagehook ] = $this->tab['layout_columns'];
			}
		}
		return $columns;
    }
	
	//setup the admin menu
	function setup_admin_menu() {		
		wp_enqueue_style('style-menu-upf', plugins_url('css/menu.css', __FILE__ ));  
		
		$page = $this->page;
		if (empty($page['parent_slug'])) {
			$this->pagehook = add_menu_page($page['title'] , $page['menu_title'] , $page['capability'] , $page['slug'] , array(&$this, 'on_page_body') , $page['menu_icon'], $page['menu_position'] );
			if(!empty($page['sub_menu_title']) && $page['sub_menu_title'] != $page['menu_title'] ){
				add_submenu_page($page['slug'],$page['title'],$page['sub_menu_title'],$page['capability'] ,$page['slug'],array(&$this, 'on_page_body') );
			}
		}else{
			$this->pagehook = add_submenu_page($page['parent_slug'] , $page['title'] , $page['menu_title'] , $page['capability'] , $page['slug'], array(&$this, 'on_page_body'));
		}				
		
		add_action('load-'.$this->pagehook, array(&$this, 'on_page_load'));	
		add_action('admin_head-'.$this->pagehook, array(&$this, 'on_page_head'));	
		add_action('admin_footer-'.$this->pagehook, array(&$this, 'on_page_footer'));	
	}	

	function on_page_load() {
		$holders = $this->tab['holders']; 
		$this->create_boxes($holders, 'on_page_load');
		
		wp_enqueue_style('style-styles-upf', plugins_url('css/styles.css', __FILE__ ));  	
		wp_enqueue_script('script-scripts-upf', plugins_url('js/scripts.js', __FILE__ ));  
			
		wp_enqueue_script('common');
		wp_enqueue_script('wp-lists');
		wp_enqueue_script('postbox');
		wp_enqueue_script('dashboard');

		wp_enqueue_script('jquery');    
		wp_enqueue_script('jquery-ui-core');   
		wp_enqueue_script('jquery-ui-custom-upf', plugins_url('js/jquery-ui-1.10.0.custom.min.js', __FILE__ ), array( 'jquery-ui-core' ), false, false );	
		wp_enqueue_script('jquery-validate-upf', plugins_url('js/jquery.validate.js', __FILE__ ), array( 'jquery' ), false, false );		
		
		wp_enqueue_style('jquery-ui-theme-upf', plugins_url('css/jquery-ui-theme/jquery-ui-1.10.0.custom.css', __FILE__ ));
		wp_enqueue_style('jquery-ui-theme-upf', plugins_url('css/jquery-ui-theme/jquery.ui.1.10.0.ie.css', __FILE__ ));		
		wp_enqueue_script('jquery-ui-slider-upf', plugins_url('js/slider.js', __FILE__ ), array( 'jquery-ui-custom-upf' ), false, false );	
		wp_enqueue_script('jquery-ui-datepicker-upf', plugins_url('js/datepicker.js', __FILE__ ), array( 'jquery-ui-custom-upf' ), false, false );		
		wp_enqueue_script('jquery-ui-progressbar-upf', plugins_url('js/progressbar.js', __FILE__ ), array( 'jquery-ui-custom-upf' ), false, false );		

		wp_enqueue_script('wp-color-picker-upf', plugins_url('js/colorpicker.js', __FILE__ ), array( 'wp-color-picker' ), false, true );		
		wp_enqueue_script('wp-color-picker');	
		wp_enqueue_style('wp-color-picker');	
	}

	function on_page_head(){
		$holders = $this->tab['holders']; 
		$this->create_boxes($holders, 'on_page_head');
	}

	function on_page_footer(){
		$holders = $this->tab['holders'];  
		$this->create_boxes($holders, 'on_page_footer');
		?>
		<script type="text/javascript">
			//<![CDATA[
			jQuery(document).ready( function($) {
				// close postboxes that should be closed
				$('.if-js-closed').removeClass('if-js-closed').addClass('closed');
				// postboxes setup
				postboxes.add_postbox_toggles('<?php echo $this->pagehook; ?>');
			});
			jQuery(document).ready( function($) {
				// close postboxes that should be closed
				$('.if-js-closed').removeClass('if-js-closed').addClass('closed');
				// postboxes setup
				postboxes.add_postbox_toggles('<?php echo $this->pagehook; ?>');
			});
			//]]>
		</script>
		<?php
	}
	
	function on_page_body() {			
		echo '<div class="wrap upf">';

			$icon = $this->child['icon'];
			if(!empty($icon)){
				echo screen_icon( $icon );
			}
			
			$heading = $this->child['heading'];
			$buttons = $this->child['buttons'];
			$this->create_heading($heading,$buttons);

			$tabs = $this->child['tabs'];
			$this->create_tabs($tabs);

			$notices = $this->page['notices'];
			$this->create_notices($notices);

			echo '<form action="" method="post" class="validate">';			
				
				wp_nonce_field('admin-metabox-general'); // save metaboxes 
				wp_nonce_field('closedpostboxes', 'closedpostboxesnonce', false ); // save closed metaboxes
				wp_nonce_field('meta-box-order', 'meta-box-order-nonce', false ); // save order metaboxes 

				$holders = $this->tab['holders']; 
				$this->create_layout($holders);
				
			echo '</form>';
		echo '</div>';
	}	

	function on_page_save() {
		$holders = $this->tab['holders'];  
		$this->create_boxes($holders, 'on_page_save');
				
		$location = $this->create_url('tab').'&notice=updated';	
		wp_redirect($location);
		exit();
	}
	
	function on_page_delete($option, $redirect = true) {
		//$holders = $this->tab['holders'];  
		//$this->create_boxes($holders, 'on_page_delete');	
		global $wpdb;
		$crumble = explode ( '_' , $option );
		$page = $crumble['0'];
		$child = $crumble['1'];
		$item = $crumble['2'];

		
		if($child == 'client'){
			//remove user
			//$field['value'] = upf_get_option('clients_client_'.$this->item['id'].'_information_user');			
			//wp_delete_user($field['value']['user_id']);
			
			//remove contacts
			$where  = " WHERE page_id = 'clients'";
			$where .= " AND child_id = 'contact'"; 
			$where .= " AND tab_id = 'information'";
			$where .= " AND field_id = 'company'";
			$where .= " AND meta_value = '".$item."'";	
			
			$sql = "SELECT item_id FROM ".$wpdb->base_prefix."upf_options ".$where."";
			$results_obj = $wpdb->get_results($sql);		
			foreach($results_obj as $key => $result){
				upf_delete_options('clients_contact_'.$result->item_id);
			}
	
			//remove projects
			$where  = " WHERE page_id = 'projects'";
			$where .= " AND child_id = 'project'"; 
			$where .= " AND tab_id = 'information'";
			$where .= " AND field_id = 'client'";
			$where .= " AND meta_value = '".$item."'";	
			
			$sql = "SELECT item_id FROM ".$wpdb->base_prefix."upf_options ".$where."";
			$results_obj = $wpdb->get_results($sql);		
			foreach($results_obj as $key => $result){
				//remove tasks
				$where  = " WHERE page_id = 'projects'";
				$where .= " AND child_id = 'task'"; 
				$where .= " AND tab_id = 'information'";
				$where .= " AND field_id = 'project'";
				$where .= " AND meta_value = '".$result->item_id."'";	
				
				$s_sql = "SELECT item_id FROM ".$wpdb->base_prefix."upf_options ".$where."";
				$s_results_obj = $wpdb->get_results($s_sql);		
				foreach($s_results_obj as $s_key => $s_result){
					upf_delete_options('projects_task_'.$s_result->item_id);
				}	
				upf_delete_options('projects_project_'.$result->item_id);
			}
				
			//remove invoices
			$where  = " WHERE page_id = 'invoices'";
			$where .= " AND child_id = 'invoice'"; 
			$where .= " AND tab_id = 'information'";
			$where .= " AND field_id = 'client'";
			$where .= " AND meta_value = '".$item."'";	
			
			$sql = "SELECT item_id FROM ".$wpdb->base_prefix."upf_options ".$where."";
			$results_obj = $wpdb->get_results($sql);		
			foreach($results_obj as $key => $result){
				//remove tasks
				$where  = " WHERE page_id = 'invoices'";
				$where .= " AND child_id = 'item'"; 
				$where .= " AND tab_id = 'information'";
				$where .= " AND field_id = 'invoice'";
				$where .= " AND meta_value = '".$result->item_id."'";	
				
				$s_sql = "SELECT item_id FROM ".$wpdb->base_prefix."upf_options ".$where."";
				$s_results_obj = $wpdb->get_results($s_sql);	
				foreach($s_results_obj as $s_key => $s_result){
					upf_delete_options('invoices_item_'.$s_result->item_id);
				}	
				upf_delete_options('invoices_invoice_'.$result->item_id);
			}
			
		}elseif($child == 'project'){
			//remove tasks
			$where  = " WHERE page_id = 'projects'";
			$where .= " AND child_id = 'task'"; 
			$where .= " AND tab_id = 'information'";
			$where .= " AND field_id = 'project'";
			$where .= " AND meta_value = '".$item."'";	
			
			$sql = "SELECT item_id FROM ".$wpdb->base_prefix."upf_options ".$where."";
			$results_obj = $wpdb->get_results($sql);		
			foreach($results_obj as $key => $result){
				upf_delete_options('projects_task_'.$result->item_id);
			}
		}elseif($child == 'invoice'){
			//remove items
			$where  = " WHERE page_id = 'invoices'";
			$where .= " AND child_id = 'item'"; 
			$where .= " AND tab_id = 'information'";
			$where .= " AND field_id = 'invoice'";
			$where .= " AND meta_value = '".$item."'";	
			
			$sql = "SELECT item_id FROM ".$wpdb->base_prefix."upf_options ".$where."";
			$results_obj = $wpdb->get_results($sql);		
			foreach($results_obj as $key => $result){
				upf_delete_options('invoices_item_'.$result->item_id);
			}			
		}		
		
		$deleted = upf_delete_options( $option );
		if($redirect){
			$location = $this->create_url('page').'&notice=deleted';	
			wp_redirect($location);
			exit;
		}
		
	}
	
	function create_heading($heading = '',$buttons = array()) {		

		if(!empty($heading)){
			echo '<h2>'.$heading.' ';
		}		
		if(is_array($buttons)){
			foreach ($buttons as $button) {			
				if(!empty($button['url'])){
					echo '<a href="'.$this->create_url($button['url_prefix']).$button['url'].'" class="add-new-h2" target="'.$button['target'].'">'.$button['title'].'</a>';
				}
			}
		}		
		if(!empty($heading)){
			echo '</h2>';
		}
	}		

	function create_tabs($tabs) {
		if(isset($_GET['action']) && !empty($_GET['action'])){
			$action = '&action='.esc_html($_GET['action']);
		}else{
			$action = '';
		}
		if(is_array($tabs)){		
			foreach ($tabs as $tab) {
				// select the active tab
				$active_class = ( $this->tab['slug'] == $tab['slug'] ) ? 'nav-tab-active' : '';		
				$url = ( $tab['url']  != "") ? $tab['url'] : $this->create_url('item').'&tab='. $tab['slug'] . $action ;
				$navtabs[] = '<a id="'.$this->create_prefix('item') . $tab['slug'] . '-tab" class="nav-tab '. $active_class .'" title="' . $tab['title'] . '" href="'.$url.'">' . $tab['title'] . '</a>';
			}
			if(count($navtabs) > 1){ 
				echo '<h2 class="nav-tab-wrapper">'.implode($navtabs).'</h2>'; 
			}
		}
	}	

	function create_notices($custom_notices) {
		$default_notices = array(
			array(
				"title" => "updated",
				"genre" => "updated",
				"message" => "All changes are updated. Go and check results!"
			),
			array(
				"title" => "deleted",
				"genre" => "updated",
				"message" => "Item succesfull deleted. Go and check results!"
			),
			array(
				"title" => "m_deleted",
				"genre" => "updated",
				"message" => "Item(s) succesfull deleted. Go and check results!"
			),
			array(
				"title" => "error",
				"genre" => "error",
				"message" => "No changes were saved. Go back and try again!"	
			)
		);
		if(is_array($custom_notices)){
			$notices = array_merge($default_notices, $custom_notices);
		}else{
			$notices = $default_notices;
		}
		 
		// Check delete action of WP_List_table
		if(isset( $_REQUEST['action'] ) && $_REQUEST['action'] === 'delete' ){
			$_GET['notice'] = 'm_deleted';
		}
		if(isset( $_REQUEST['action2'] ) && $_REQUEST['action2'] === 'delete' ){
			$_GET['notice'] = 'm_deleted';
		}
		if(is_array($notices)){
			foreach ($notices as $notice) {
			   $_GET['notice'] = isset($_GET['notice']) ? $_GET['notice'] : '';
				if ($notice['title'] == $_GET['notice']) {
						echo "<div id='message' class='".$notice['genre']."'><p>".$notice['message']."</p></div>";
				}
			}		
		}		
	}

	function create_layout($holders){	
		//we need the global screen column value to beable to have a sidebar in WordPress 2.8
		global $screen_layout_columns;
		$post = '';
		// set the screen layout columns to the maximun supported columns if the user had selected more than is supported
		if($screen_layout_columns > 0){
			$screen_layout_columns =  $screen_layout_columns > $this->tab['layout_columns'] ? $this->tab['layout_columns'] : $screen_layout_columns;
		}else{
			$screen_layout_columns = $this->tab['layout_columns'];
		}

		$wrap_id = $this->tab['layout'] == 'widgets' ? 'dashboard-widgets-wrap' : 'poststuff';		
		$body_id = $this->tab['layout'] == 'widgets' ? 'dashboard-widgets' : 'post-body';		
		
		$mainholder = false;
		if(is_array($holders)){
			foreach ($holders as $holder) {
				if($holder['context'] == "main"){
					$mainholder = true;
				}
			}		
		}			
		
		if($mainholder == true && $this->tab['layout'] == "widgets"){
			echo '<div id="poststuff" class="plfw">';
				echo'<div id="post-body" class="">';
					echo '<div id="post-body-content">';
						$this->create_boxes($holders, 'on_page_body', array('main'));				
					echo '</div>';
				echo '</div>';
			echo '</div>';
		}
		
		echo '<div id="'.$wrap_id.'" class="plfw">';
			echo'<div id="'.$body_id.'" class="metabox-holder columns-'.$screen_layout_columns.'">';
				if($mainholder == true && $this->tab['layout'] == "post"){
					echo '<div id="post-body-content">';
						$this->create_boxes($holders, 'on_page_body', array('main'));				
					echo '</div>';
				}
				echo '<div id="postbox-container-1" class="postbox-container">';
					$this->create_boxes($holders, 'on_page_body', array('side'));
					do_meta_boxes($this->pagehook, 'side',$post);	
				echo '</div>';
				echo '<div id="postbox-container-2" class="postbox-container">';
					$this->create_boxes($holders, 'on_page_body', array('normal'));
					do_meta_boxes($this->pagehook, 'normal',$post);
				echo '</div>';
				echo '<div id="postbox-container-3" class="postbox-container">';
					$this->create_boxes($holders, 'on_page_body', array('advanced'));
					do_meta_boxes($this->pagehook, 'advanced',$post);
				echo '</div>';
				echo '<div id="postbox-container-4" class="postbox-container">';
					$this->create_boxes($holders, 'on_page_body', array('additional'));
					do_meta_boxes($this->pagehook, 'additional',$post);						
				echo '</div>';
			echo '</div>';
		echo '</div>';
	}
	
	function create_boxes($holders, $class_function = 'on_page_body', $contexts = '' ){
		$holder_select = true;
		if(!is_array($contexts)){
			$contexts = array();
			$holder_select = false;
		}
		if(is_array($holders)){
			foreach ($holders as $holder) {
				if($holder_select == false || in_array($holder['context'],$contexts)){
					$boxes = $holder['boxes'];
					if(is_array($boxes)){
						foreach ($boxes as $box) {
							$box['context'] = $holder['context'];
							$this->boxes[$box['type']]->$class_function($box);
						}
					}
				}
			}
		}		
	}
	
	function create_url($depth = 'page', $absoluut = true){
		$url = '';
		
		if(isset($_GET['page']) && !empty($_GET['page'])){
			$params['page'] = '?page='.esc_html($_GET['page']);
		}else{
			$params['page'] = '';
		}
		if(isset($_GET['child']) && !empty($_GET['child'])){
			$params['child'] = '&child='.esc_html($_GET['child']);
		}else{
			$params['child'] = '';
		}
		if(isset($_GET['tab']) && !empty($_GET['tab'])){
			$params['tab'] = '&tab='.esc_html($_GET['tab']);
		}else{
			$params['tab'] = '';
		}
		if(isset($_GET['item']) && !empty($_GET['item'])){
			$params['item'] = '&item='.esc_html($_GET['item']);
		}else{
			$params['item'] = '';
		}
		if($absoluut == true){
			$prefix = admin_url('admin.php');
		}
		switch($depth){
			case 'page';
				$url = $prefix.$params['page'];
			break;			
			case 'child';
				$url = $prefix.$params['page'].$params['child'];
			break;			
			case 'item';
				$url = $prefix.$params['page'].$params['child'].$params['item'];
			break;
			case 'tab';
				$url = $prefix.$params['page'].$params['child'].$params['item'].$params['tab'];
			break;
		}
		return $url;		
	}
	
	function create_prefix($depth = 'page', $seperator = '-'){
		switch($depth){
			case 'page';
				$prefix = $this->page['slug'].$seperator;
			break;			
			case 'child';
				$prefix = $this->page['slug'].$seperator.$this->child['slug'].$seperator;
			break;			
			case 'item';
				$prefix = $this->page['slug'].$seperator.$this->child['slug'].$seperator.$this->item['id'].$seperator;
			break;
			case 'tab';
				$prefix = $this->page['slug'].$seperator.$this->child['slug'].$seperator.$this->item['id'].$seperator.$this->tab['slug'].$seperator;
			break;
			case 'field';
				$prefix = $this->page['slug'].$seperator.$this->child['slug'].$seperator.'#'.$seperator.$this->tab['slug'].$seperator;
			break;
		}
		return $prefix;
	}
	
	function on_activation() {
		$plugin_options = maybe_unserialize(get_option($this->plugin_option_name));
				
		if ( false === $plugin_options ) {
			// Install plugin
			global $wpdb;
			
			$table_name = $wpdb->prefix . 'upf_options';
			$sql = "
				CREATE TABLE ".$table_name." (
				  meta_id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
				  page_id varchar(255) NOT NULL,
				  child_id varchar(255) NOT NULL,
				  item_id varchar(255) NOT NULL,
				  tab_id varchar(255) NOT NULL,
				  field_id varchar(255) NOT NULL,
				  meta_key varchar(255) DEFAULT NULL,
				  meta_value longtext,
				  PRIMARY KEY (meta_id),
				  KEY meta_key (meta_key)
				) ENGINE=MyISAM  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;
			";
			
			require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
			dbDelta( $sql );
			
			$plugin_options['version'] = $this->version;	
			$plugin_options['dir'] = $this->dir;
			$plugin_options['path'] = $this->path;
			
			add_option($this->plugin_option_name,maybe_serialize($plugin_options));
			add_role('client', 'Client', array(
				'read' => false, // True allows that capability, False specifically removes it.
				'edit_posts' => false,
				'delete_posts' => false,
				'edit_published_posts' => false,
				'publish_posts' => false,
				'level' => '0',
			));	
			
			// set capabilities
			$role = get_role( 'administrator' );
			$role->add_cap( 'client_manager_clients' ); 
			$role->add_cap( 'client_manager_projects' ); 
			$role->add_cap( 'client_manager_invoices' ); 
			$role->add_cap( 'client_manager_settings' ); 
				
			$header = '<div class="company-address"><strong>[company]</strong><br>[company_street]<br>[company_city]<br>[company_zipcode]<br>[company_phone]</div><br><div class="recipient-address"><strong>[client]</strong><br>[client_street]<br>[client_city]<br>[client_zipcode]<br>[client_phone]</div><br><div class="clear"></div><br><h1>Invoice [invoicenumber]</h1><br><p class="terms"><strong>[createdate]</strong><br>Payment due by [duedate]</p>';
			if(!upf_get_option('settings_settings_1_invoices_header')){
				upf_update_option('settings_settings_1_invoices_header', $header);
			}
			if(!upf_get_option('settings_settings_1_invoices_footer')){
				$conditions = upf_get_option('settings_settings_1_invoices_conditions');
				$thanks = upf_get_option('settings_settings_1_invoices_thanks');
				$footer = $conditions.'<hr>'.$thanks;
			
				upf_update_option('settings_settings_1_invoices_footer', $footer);
			}
			
		} else if ( $this->version != $plugin_options['version'] ) {
			// Upgrade plugin
			global $wpdb;
			
			$table_name = $wpdb->prefix . 'upf_options';
			$sql = "
				CREATE TABLE IF NOT EXISTS ".$table_name." (
				  meta_id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
				  page_id varchar(255) NOT NULL,
				  child_id varchar(255) NOT NULL,
				  item_id varchar(255) NOT NULL,
				  tab_id varchar(255) NOT NULL,
				  field_id varchar(255) NOT NULL,
				  meta_key varchar(255) DEFAULT NULL,
				  meta_value longtext,
				  PRIMARY KEY (meta_id),
				  KEY meta_key (meta_key)
				) ENGINE=MyISAM  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;
			";
			
			require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
			dbDelta( $sql );
			
			$plugin_options['version'] = $this->version;	
			$plugin_options['dir'] = $this->dir;
			$plugin_options['path'] = $this->path;
			
			update_option($this->plugin_option_name,maybe_serialize($plugin_options));
			add_role('client', 'Client', array(
				'read' => false, // True allows that capability, False specifically removes it.
				'edit_posts' => false,
				'delete_posts' => false,
				'edit_published_posts' => false,
				'publish_posts' => false,
				'level' => '0',
			));	

			// set capabilities
			$role = get_role( 'administrator' );
			$role->add_cap( 'client_manager_clients' ); 
			$role->add_cap( 'client_manager_projects' ); 
			$role->add_cap( 'client_manager_invoices' ); 
			$role->add_cap( 'client_manager_settings' ); 			
			
			$header = '<div class="company-address"><strong>[company]</strong><br>[company_street]<br>[company_city]<br>[company_zipcode]<br>[company_phone]</div><br><div class="recipient-address"><strong>[client]</strong><br>[client_street]<br>[client_city]<br>[client_zipcode]<br>[client_phone]</div><br><div class="clear"></div><br><h1>Invoice [invoicenumber]</h1><br><p class="terms"><strong>[createdate]</strong><br>Payment due by [duedate]</p>';
			if(!upf_get_option('settings_settings_1_invoices_header')){
				upf_update_option('settings_settings_1_invoices_header', $header);
			}
			if(!upf_get_option('settings_settings_1_invoices_footer')){
				$conditions = upf_get_option('settings_settings_1_invoices_conditions');
				$thanks = upf_get_option('settings_settings_1_invoices_thanks');
				$footer = $conditions.'<hr>'.$thanks;
			
				upf_update_option('settings_settings_1_invoices_footer', $footer);
			}
		}
	}

}

include_once("pages.php");
if(is_array($pages)){
	foreach ($pages as $page) {
		$upf = new upf($page);
	}	
}

register_activation_hook( __FILE__, 'upf_register_activation');		
function upf_register_activation(){
	$upf = new upf(false);
}