<?php
//avoid direct calls to this file where wp core files not present
if (!function_exists ('add_action')) {
	header('Status: 403 Forbidden');
	header('HTTP/1.1 403 Forbidden');
	exit();
}

//adding the id var so that WP recognizes it
add_filter( 'query_vars','upf_insert_query_vars' );
function upf_insert_query_vars( $vars ){
    array_push($vars, 'ipn_handler');
    return $vars;
}

//flush_rules() if our rules are not yet included
add_action( 'wp_loaded','upf_flush_rules' );
function upf_flush_rules(){
	$rules = get_option( 'rewrite_rules' );
	if ( ! isset( $rules['ipn_handler/paypal'] ) ) {
		global $wp_rewrite;
	   	$wp_rewrite->flush_rules();
	}
}

//adding a new rule
add_action('generate_rewrite_rules', 'upf_rewrite_rules');
function upf_rewrite_rules( $wp_rewrite ) {
  $new_rules = array('ipn_handler/paypal' => 'index.php?ipn_handler=paypal');
  $wp_rewrite->rules = $new_rules + $wp_rewrite->rules;
}

//trigger IPN
add_action('parse_request', 'upf_parse_request');
function upf_parse_request($wp) {
    // only process requests with "ipn_handler=paypal"
    if (array_key_exists('ipn_handler', $wp->query_vars) && $wp->query_vars['ipn_handler'] == 'paypal') {
		upf_ipn_handler($wp);
    }
} 

// upf_plugin_proccess_paypal_ipn
function upf_ipn_handler($wp){

	$receiver_email = upf_get_option('settings_settings_1_invoices_paypalemail');
	$debug_email = upf_get_option('settings_settings_1_invoices_paypaldebug');
	if(empty($debug_email)){
		$debug_email = false;
	}	
	
	$paypalsandbox = upf_get_option('settings_settings_1_invoices_paypalsandbox');
	if($paypalsandbox['option'] != '1'){
		$sandbox_mode = false;
	}else{
		$sandbox_mode = true;
	}
	
	$invoice_key = 'invoices_invoice_'.esc_html($_POST['invoice']).'_';
	$invoice = upf_get_options($invoice_key);	
	
	// tell PHP to log errors to ipn_errors.log in this directory
	ini_set('log_errors', true);
	ini_set('error_log', dirname(__FILE__).'/ipn_errors.log');

	// intantiate the IPN listener
	include('ipnlistener.php');
	$listener = new IpnListener();

	// tell the IPN listener to use the PayPal test sandbox
	$listener->use_curl = false;
	$listener->use_sandbox = $sandbox_mode;
	
	// try to process the IPN POST
	try {
		$listener->requirePostMethod();
		$verified = $listener->processIpn();
	} catch (Exception $e) {
		error_log($e->getMessage());
		exit(0);
	}
	// TODO: Handle IPN Response here
	if ($verified) {
		
		$errmsg = '';   // stores errors from fraud checks
		
		// 1. Make sure the payment status is "Completed" 
		if ($_POST['payment_status'] != 'Completed') { 
			// simply ignore any IPN that is not completed
			exit(0); 
		}
		
		// 2. Make sure seller email matches your primary account email.
		if ($_POST['receiver_email'] != $receiver_email) {
			$errmsg .= "'receiver_email' does not match: ";
			$errmsg .= $_POST['receiver_email']." != ".$receiver_email."\n";
		}
		
		// 3. Make sure the currency code matches
		$currency = isset($invoice[$invoice_key.'information_currency']) ? $invoice[$invoice_key.'information_currency'] : '';
		$currency = maybe_unserialize($currency);
		if(!empty($currency['option'])){
			$currency_code = $currency['option'];
		}else{
			$currency_code = 'USD';
		}		
		if ($_POST['mc_currency'] != $currency_code) {
			$errmsg .= "'mc_currency' does not match: ";
			$errmsg .= $_POST['mc_currency']." != ".$currency_code."\n";
		}		
		
		$due = isset($invoice[$invoice_key.'information_due']) ? $invoice[$invoice_key.'information_due'] : '';		
		// 4. Make sure the amount(s) paid match
		if ($_POST['mc_gross'] != $due) {
			$errmsg .= "'mc_gross' does not match: ";
			$errmsg .= $_POST['mc_gross']." != ".$due."\n";
		}
		
		$paypal = isset($invoice[$invoice_key.'information_paypal']) ? $invoice[$invoice_key.'information_paypal'] : '';	
		$paypal = maybe_unserialize($paypal);
		$txn_id = $paypal['txn_id'];
		// 5. Make sure this is not a duplicate txn_id
		if ($_POST['txn_id'] == $txn_id) {
			$errmsg .= "'txn_id' has already been processed: ".$_POST['txn_id']."\n";
		}
			
		if (!empty($errmsg)) {
		
			// manually investigate errors from the fraud checking
			$body = "IPN failed fraud checks: \n$errmsg\n\n";
			$body .= $listener->getTextReport();
			mail($debug_email, 'IPN Fraud Warning', $body);
			
		} else {		
			// TODO: process order here
			$field['status']['option'] = 'paid';		
			$field['status']['title'] = 'Paid';
						
			$field['status'] = serialize($field['status'] );
			upf_update_option($invoice_key.'information_status', $field['status']);	

			$old = upf_get_option($invoice_key.'information_paypal');	
			$new = $_POST;
			if(is_array($old)){
				$field['paypal'] = array_merge($old, array($new));
			}else{
				$field['paypal'] = array($new);
			}
			$field['paypal'] = serialize($field['paypal'] );
			upf_update_option($invoice_key.'information_paypal', $field['paypal']);	

			upf_update_option($invoice_key.'information_due', '0');				
		}
		
	} else {
		// manually investigate the invalid IPN
		mail($debug_email, 'Invalid IPN', $listener->getTextReport());
	}

}
?>